package org.gvsig.raster.projecttemplate.algorithm;

import java.util.HashMap;

import javax.swing.SwingUtilities;

import org.gvsig.fmap.dal.coverage.RasterLocator;
import org.gvsig.fmap.dal.coverage.dataset.Buffer;
import org.gvsig.fmap.dal.coverage.exception.InvalidSetViewException;
import org.gvsig.fmap.dal.coverage.exception.ProcessInterruptedException;
import org.gvsig.fmap.dal.coverage.exception.RasterDriverException;
import org.gvsig.fmap.dal.coverage.store.RasterDataStore;
import org.gvsig.fmap.dal.coverage.store.RasterQuery;
import org.gvsig.i18n.Messages;
import org.gvsig.raster.tools.algorithm.base.RasterBaseAlgorithmLibrary;
import org.gvsig.raster.tools.algorithm.base.process.RasterProcess;

/**
 * Process 
 */
public class ProjectTemplateProcess extends RasterProcess {
	public static String      RASTER_STORE1     = "RasterStore1";
	public static String      PATH              = "Path";
	public static String      FILENAME          = "FileName";
	public static String      TIME              = "Time";
	
	private RasterDataStore   store             = null;
	private String            filename          = null;
	private long              millis            = 0;
	
	public static void registerParameters() {
		RASTER_STORE1 = RasterBaseAlgorithmLibrary.registerInputParameter(RASTER_STORE1, RasterDataStore.class);
		PATH = RasterBaseAlgorithmLibrary.registerInputParameter(PATH, String.class);
		FILENAME = RasterBaseAlgorithmLibrary.registerOutputParameter(FILENAME, String.class);
		TIME = RasterBaseAlgorithmLibrary.registerOutputParameter(TIME, Long.class);
	}
	
	public void init() {
		store = getParam(RASTER_STORE1) != null ? (RasterDataStore)getParam(RASTER_STORE1) : null;
		filename = getStringParam(PATH);
	}

	public void process() throws ProcessInterruptedException {
		long t1 = new java.util.Date().getTime();
		insertLineLog(Messages.getText("..."));
		try {
			if (store == null)
				throw new ProjectTemplateException("...");
			
			int inDataType = store.getDataType()[0];
			Buffer buf = RasterLocator.getManager().createBuffer(
					inDataType, 
					(int)store.getWidth(), 
					(int)store.getHeight(), 
					store.getBandCount(), 
					true);
			
			RasterQuery query = RasterLocator.getManager().createQuery();
			query.setAllDrawableBands();
			query.setAreaOfInterest();
			Buffer sourceBuffer = null;
			try {
				sourceBuffer = store.query(query);
			} catch (RasterDriverException e) {
				throw new ProjectTemplateException("");
			} catch (InvalidSetViewException e) {
				throw new ProjectTemplateException("");
			}
			
			//updatePercent(row, buf.getHeight());
			
			//super.exportRaster(filename, buf, cellsize, ulx, uly);
			
			long t2 = new java.util.Date().getTime();
			millis = t2 - t1;
			
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					if (externalActions != null) {
						HashMap<String, Object> map = new HashMap<String, Object>();
						map.put(FILENAME, filename);
						map.put(TIME, new Long(millis));
						externalActions.end(map);
					}
				}
			});
		} catch (ProjectTemplateException e) {
			if (incrementableTask != null)
				incrementableTask.processFinalize();
			messageBoxError("...", this, e);
		}
	}
	
	public Object getResult() {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put(FILENAME, filename);
		map.put(TIME, new Long(millis));
		return map;
	}

	public int getPercent() {
		return percent;
	}

	public String getTitle() {
		return Messages.getText("...");
	}
}