package org.gvsig.raster.projecttemplate.lib;

import org.gvsig.tools.locator.BaseLocator;
import org.gvsig.tools.locator.Locator;
import org.gvsig.tools.locator.LocatorException;

/**
 * This locator is the entry point for the Raster library, providing
 * access to all Raster services through the {@link ProjectTemplateManager}
 * .
 * 
 * @author gvSIG team
 * @version $Id$
 */
public class ProjectTemplateLocator extends BaseLocator {
    public static final String                      MANAGER_NAME         = "ProjectTemplate.manager";
    public static final String                      MANAGER_DESCRIPTION  = "ProjectTemplate Manager";
    private static final String                     LOCATOR_NAME         = "ProjectTemplate.locator";
    private static final ProjectTemplateLocator  INSTANCE             = new ProjectTemplateLocator();

    /**
     * Return the singleton instance.
     * 
     * @return the singleton instance
     */
    public static ProjectTemplateLocator getInstance() {
        return INSTANCE;
    }

    /**
     * Return the Locator's name.
     * 
     * @return a String with the Locator's name
     */
    public final String getLocatorName() {
        return LOCATOR_NAME;
    }

    /**
     * Return a reference to the ProjectTemplateManager.
     * 
     * @return a reference to the ProjectTemplateManager
     * @throws LocatorException
     *             if there is no access to the class or the class cannot be
     *             instantiated
     * @see Locator#get(String)
     */
    public static ProjectTemplateManager getManager() throws LocatorException {
        return (ProjectTemplateManager) getInstance().get(MANAGER_NAME);
    }

    /**
     * Registers the Class implementing the RasterManager interface.
     * 
     * @param clazz
     *            implementing the RasterManager interface
     */
    public static void registerManager(
        Class<? extends ProjectTemplateManager> clazz) {
        getInstance().register(MANAGER_NAME, MANAGER_DESCRIPTION, clazz);
    }

}
