package org.gvsig.raster.projecttemplate.algorithm;

import java.util.HashMap;

import javax.swing.SwingUtilities;

import org.gvsig.fmap.dal.coverage.RasterLocator;
import org.gvsig.fmap.dal.coverage.dataset.Buffer;
import org.gvsig.fmap.dal.coverage.datastruct.Extent;
import org.gvsig.fmap.dal.coverage.exception.InvalidSetViewException;
import org.gvsig.fmap.dal.coverage.exception.ProcessInterruptedException;
import org.gvsig.fmap.dal.coverage.exception.RasterDriverException;
import org.gvsig.fmap.dal.coverage.store.RasterDataStore;
import org.gvsig.fmap.dal.coverage.store.RasterQuery;
import org.gvsig.i18n.Messages;
import org.gvsig.raster.tools.algorithm.base.RasterBaseAlgorithmLibrary;
import org.gvsig.raster.tools.algorithm.base.process.RasterProcess;

/**
 * Process 
 */
public class ProjectTemplateProcess extends RasterProcess {
	public static String      RASTER_STORE1     = "RasterStore1";
	public static String      BUFFER            = "RasterStore1";
	public static String      PATH              = "Path";
	public static String      FILENAME          = "FileName";
	public static String      TIME              = "Time";
	
	public static String      TEST_EXTENT       = "TestExtent";
	public static String      TEST_WIDTH        = "TestWidth";
	public static String      TEST_HEIGHT       = "TestHeight";
	
	private RasterDataStore   store             = null;
	private String            filename          = null;
	private long              millis            = 0;
	
	private Extent            testExtent        = null;
	private int               testWidth         = 0;
	private int               testHeight        = 0;
	
	/**
	 * This buffer is just to test
	 */
	private Buffer            bufferForTest     = null;
	
	public static void registerParameters() {
		RASTER_STORE1 = RasterBaseAlgorithmLibrary.registerInputParameter(RASTER_STORE1, RasterDataStore.class);
		PATH = RasterBaseAlgorithmLibrary.registerInputParameter(PATH, String.class);
		FILENAME = RasterBaseAlgorithmLibrary.registerOutputParameter(FILENAME, String.class);
		TIME = RasterBaseAlgorithmLibrary.registerOutputParameter(TIME, Long.class);
		
		TEST_EXTENT = RasterBaseAlgorithmLibrary.registerOutputParameter(TEST_EXTENT, Extent.class);
		TEST_WIDTH = RasterBaseAlgorithmLibrary.registerOutputParameter(TEST_WIDTH, Integer.class);
		TEST_HEIGHT = RasterBaseAlgorithmLibrary.registerOutputParameter(TEST_HEIGHT, Integer.class);
	}
	
	public void init() {
		store = getParam(RASTER_STORE1) != null ? (RasterDataStore)getParam(RASTER_STORE1) : null;
		filename = getStringParam(PATH);
		
		testExtent = getParam(TEST_EXTENT) != null ? (Extent)getParam(TEST_EXTENT) : null;
		testWidth = getIntParam(TEST_WIDTH);
		testHeight = getIntParam(TEST_HEIGHT);
	}

	public void process() throws ProcessInterruptedException {
		long t1 = new java.util.Date().getTime();
		insertLineLog(Messages.getText("..."));
		try {
			if (store == null)
				throw new ProjectTemplateException("...");
			
			
			//////////////////////////
			//TODO:PROCESS!!
			
			RasterQuery query = RasterLocator.getManager().createQuery();
			query.setAllDrawableBands();
			query.setAreaOfInterest(testExtent, testWidth, testHeight);
			Buffer sourceBuffer = null;
			try {
				sourceBuffer = store.query(query);
				sourceBuffer.setDataExtent(testExtent.toRectangle2D());
			} catch (RasterDriverException e) {
				throw new ProjectTemplateException("");
			} catch (InvalidSetViewException e) {
				throw new ProjectTemplateException("");
			}
			bufferForTest = sourceBuffer;
			//int inDataType = store.getDataType()[0];
			//Buffer buf = RasterLocator.getManager().createBuffer(
			//		inDataType, 
			//		(int)store.getWidth(), 
			//		(int)store.getHeight(), 
			//		store.getBandCount(), 
			//		true);
			//updatePercent(row, buf.getHeight());
			//super.exportRaster(filename, buf, cellsize, ulx, uly);
			//////////////////////////
			
			
			long t2 = new java.util.Date().getTime();
			millis = t2 - t1;
			
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					if (externalActions != null) {
						externalActions.end(getResult());
					}
				}
			});
		} catch (ProjectTemplateException e) {
			if (incrementableTask != null)
				incrementableTask.processFinalize();
			messageBoxError("...", this, e);
		}
	}
	
	public Object getResult() {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put(FILENAME, filename);
		map.put(TIME, new Long(millis));
		map.put(BUFFER, bufferForTest);
		return map;
	}

	public int getPercent() {
		return percent;
	}

	public String getTitle() {
		return Messages.getText("...");
	}
}