package org.gvsig.raster.projecttemplate.app;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.PluginServices;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.andami.ui.mdiManager.IWindow;
import org.gvsig.app.project.documents.view.gui.AbstractViewPanel;
import org.gvsig.fmap.dal.coverage.RasterLocator;
import org.gvsig.fmap.dal.coverage.util.FileUtils;
import org.gvsig.fmap.dal.exception.ReadException;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.FLayers;
import org.gvsig.fmap.mapcontrol.MapControl;
import org.gvsig.i18n.Messages;
import org.gvsig.raster.fmap.layers.FLyrRaster;
import org.gvsig.raster.projecttemplate.swing.ProjectTemplatePanel;
import org.gvsig.raster.projecttemplate.swing.ProjectTemplateSwingLocator;
import org.gvsig.raster.swing.RasterSwingLocator;
import org.gvsig.raster.swing.newlayer.CreateNewLayerPanel;
import org.gvsig.raster.swing.newlayer.FileNameManagement;
import org.gvsig.raster.swing.preview.PreviewPanel;
import org.gvsig.raster.tools.algorithm.swing.bean.LayerComboElement;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.extensionpoint.ExtensionPoint;
import org.gvsig.tools.extensionpoint.ExtensionPointManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class ProjectTemplateExtension extends Extension implements ActionListener {
	private int                      windowWidth         = 600;
	private int                      windowHeight        = 350;
	private IWindow                  window              = null;
	private PreviewPanel             previewPanel        = null;
	private ProjectTemplatePreview   processPreview      = null;
	private ProjectTemplatePanel     panel               = null;
	private CreateNewLayerPanel      newLayerPanel       = null;
	private static final Logger      logger              = LoggerFactory.getLogger(ProjectTemplateExtension.class);
	
    public void initialize() {
    	
    	// Adds an entry to the TOC's floating menu to those layers defined in this extension
    	
		//ExtensionPoint exPoint = ToolsLocator.getExtensionPointManager().add("My_TocActions");
		//exPoint.append("MyMenuEntry", "", new ProjectTemplateTocMenuEntry());

        // Adds a new tab to the "add layer" dialog
    	
    	//AddLayer.addWizard(ProjectTemplateWizard.class);
    	
    	// Adds a new button in the raster bar
    	
		ExtensionPointManager extensionPoints = ToolsLocator.getExtensionPointManager();
		ExtensionPoint point = extensionPoints.add("GenericToolBarMenu");
		point.append("ProjectTemplate", "", ProjectTemplateTocMenuEntry.getSingleton());
		ProjectTemplateTocMenuEntry.setExtension(this);
    	
        Messages.addResourceFamily("org.gvsig.raster.projecttemplate.app.i18n.text",
        		ProjectTemplateExtension.class.getClassLoader(),
        		ProjectTemplateExtension.class.getClass().getName()); 

    	initilizeIcons();
    }
    
	public void postInitialize() {
	}

    public void execute(String actionCommand) {
    	if (actionCommand.compareTo("ProjectTemplate") == 0) {
    		if(getSelectedLayer() == null) {
    			messageBoxError("layer_not_selected", null, null);
    			return;
    		}
    		try {
    			initializeMainPanel();
    		} catch (ReadException e) {
    			logger.error("Error initializing the main panel", e);
    		}
    	}
    }
    
    /**
     * Creates and initializes the main panel
     * @throws ReadException
     */
    private void initializeMainPanel() throws ReadException {
       loadProjectTemplatePanel();
      
       MainPanel mainPanel = new MainPanel(
    		   getProjectTemplatePanel().getComponent(), 
    		   (JComponent)getNewLayerPanel());
       
       window = new ProjectTemplateWindow(
    		   mainPanel, 
    		   (JPanel)getPreviewPanel(), 
    		   Messages.getText("projecttemplate"), 
    		   windowWidth, 
    		   windowHeight,
    		   false,
    		   this);
       
       PluginServices.getMDIManager().addWindow(window);
       getPreviewPanel().previewInitializeSizes();
       getPreviewPanel().refreshPreview();
    }
    
    /**
     * Loads the main panel with the data
     */
    private void loadProjectTemplatePanel() {
    	//TODO: Load panel with the initial data
    	LayerElement element = new LayerElement(getSelectedLayer());
    	getProjectTemplatePanel().getDataModel().setInputLayer(element);
    }

    
    /**
     * Gets the list of layers from the gvSIG view
     * @return
     */
    private FLayers getLayers() {
        MapControl mapControl = null;
        IWindow[] windowList = PluginServices.getMDIManager().getAllWindows();
        for (int i = 0; i < windowList.length; i++) {
            if (windowList[i] instanceof AbstractViewPanel)
                mapControl = ((AbstractViewPanel) windowList[i]).getMapControl();
        }
        if (mapControl != null)
            return mapControl.getMapContext().getLayers();
        return null;
    }
    
    /**
     * Gets the selected layer
     * @return
     */
    private FLyrRaster getSelectedLayer() {
    	FLayers lyrs = getLayers();
    	for (int i = 0; i < lyrs.getLayersCount(); i++) {
			FLayer lyr = lyrs.getLayer(i);
			if(lyr.isActive() && lyr instanceof FLyrRaster)
				return (FLyrRaster)lyr;
		}
    	return null;
    }
    
    public boolean isEnabled() {
        return false;
    }

    public boolean isVisible() {
        return false;
    }

    void initilizeIcons() {
		IconThemeHelper.registerIcon(null, "gvsig-icon", this);
    }
    
    /**
     * Gets the {@link ProjectTemplatePanel}
     * @return {@link ProjectTemplatePanel}
     */
    private CreateNewLayerPanel getNewLayerPanel() {
    	if(newLayerPanel == null)
    		 newLayerPanel = RasterSwingLocator.getSwingManager().createNewLayerPanel(
    				 new FileNameManagementImpl(),
    				 Messages.getText("create_output_layer"));
    	return newLayerPanel;
    }
    
    /**
     * Gets the {@link PreviewPanel}
     * @return {@link PreviewPanel}
     */
    private PreviewPanel getPreviewPanel() {
    	if(previewPanel == null)
    		previewPanel = RasterSwingLocator.getSwingManager().createSinglePreviewPanel(null, getDataSourcePreview());
    	return previewPanel;
    }
    
    /**
     * Gets the {@link ProjectTemplatePreview}
     * @return {@link ProjectTemplatePreview}
     */
    private ProjectTemplatePreview getDataSourcePreview() {
    	if(processPreview == null) {
    		processPreview = new ProjectTemplatePreview();
    	}
    	processPreview.setData(getProjectTemplatePanel().getDataModel());
    	return processPreview;
    }
    
    /**
     * Gets the {@link ProjectTemplatePanel}
     * @return {@link ProjectTemplatePanel}
     */
    private ProjectTemplatePanel getProjectTemplatePanel() {
    	if(panel == null)
    		panel = ProjectTemplateSwingLocator.getSwingManager().createProjectTemplatePanel();
    	return panel;
    }

	public void actionPerformed(ActionEvent e) {
		
	}
	
	/**
	 * Shows a error dialog with a text and a accept button 
	 * @param msg Message to show in the dialog
	 * @param parentWindow Parent window
	 */
	public static void messageBoxError(String msg, Object parentWindow, Exception exception) {
		if(exception != null) {
			LoggerFactory.getLogger(ProjectTemplateExtension.class).debug(Messages.getText(msg), exception);
		}
		String string = Messages.getText("accept");
		Object[] options = {string};
		JOptionPane.showOptionDialog((Component)parentWindow,
					"<html>" + Messages.getText(msg).replaceAll("\n", "<br>") + "</html>",
					Messages.getText("confirmacion"),
					JOptionPane.OK_OPTION,
					JOptionPane.ERROR_MESSAGE,
					null,
					options,
					string);
	}
	
	public class FileNameManagementImpl implements FileNameManagement {
		private FileUtils fileUtils = RasterLocator.getManager().getFileUtils();
		
		public String getDefaultFilePath() {
			return System.getProperty("user.home");
		}

		public List<String> getPrefixSupported() {
			List<String> list = new ArrayList<String>();
			list.add("tif");
			return list;
		}

		public String getUniqueLayerName() {
			return fileUtils.getOnlyLayerName();
		}

		public void usesUniqueLayerName() {
			fileUtils.usesOnlyLayerName();
		}

	}
	
	public class LayerElement implements LayerComboElement {
		private FLayer     lyr  = null;
		public LayerElement(FLayer lyr) {
			this.lyr = lyr;
		}
		
		public FLayer getLayer() {
			return lyr;
		}
		
		public String toString() {
			return lyr.getName();
		}
		
		public int getNumOfElements() {
			if(lyr instanceof FLyrRaster)
				return ((FLyrRaster)lyr).getDataStore().getBandCount();
			return 0;
		}
	}
}
