package org.gvsig.raster.projecttemplate.app;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.HashMap;

import org.gvsig.fmap.dal.coverage.RasterLocator;
import org.gvsig.fmap.dal.coverage.dataset.Buffer;
import org.gvsig.fmap.dal.coverage.datastruct.Extent;
import org.gvsig.fmap.dal.coverage.exception.ProcessInterruptedException;
import org.gvsig.fmap.dal.coverage.grid.render.ImageDrawer;
import org.gvsig.gui.beans.imagenavigator.ImageUnavailableException;
import org.gvsig.raster.fmap.layers.FLyrRaster;
import org.gvsig.raster.projecttemplate.algorithm.ProjectTemplateProcess;
import org.gvsig.raster.projecttemplate.app.ProjectTemplateExtension.LayerElement;
import org.gvsig.raster.projecttemplate.swing.ProjectTemplateData;
import org.gvsig.raster.swing.preview.DataSourcePreview;
import org.gvsig.raster.tools.algorithm.base.RasterBaseAlgorithmLibrary;
import org.gvsig.raster.tools.algorithm.base.process.ProcessException;
import org.gvsig.raster.tools.algorithm.base.process.RasterProcess;
import org.gvsig.tools.task.Cancellable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@SuppressWarnings("deprecation")
public class ProjectTemplatePreview implements DataSourcePreview {
	private FLyrRaster              inputLyr          = null;
	private Extent                  bbox              = null;
	private ProjectTemplateData     data              = null;
	private static final Logger     logger            = LoggerFactory.getLogger(ProjectTemplatePreview.class);
	
	
	public void setData(ProjectTemplateData data) {
		this.data = data;
		loadLayers();
	}
	
	private void loadLayers() {
		if(data == null)
			return;
		Object obj = data.getInputLayer();
		if(obj != null)
			inputLyr = (FLyrRaster)((LayerElement)data.getInputLayer()).getLayer();
		if(inputLyr != null)
			bbox = inputLyr.getFullRasterExtent();		
	}
	
	public void closePreviewLayer() {
		
	}

	public double getCellSize() {
		if(inputLyr != null)
			return inputLyr.getDataStore().getCellSize();
		return 1;
	}
	
	public DataSourcePreview cloneDataSourcePreview() {
		return this;
	}

	public void popStatus() {
	}

	public void pushStatus() {
	}
	
	public void draw(BufferedImage image, Graphics2D g, double[] coords, int w, int h, Cancellable cancel, double scale) throws ImageUnavailableException {
		//Posicin de inicio dibujado dentro del Graphics
		int x = 0;
		int y = 0;
		//Coordenadas de la peticin
		double iniX = coords[0];
		double iniY = coords[1];
		double endX = coords[2];
		double endY = coords[3];
		
		//Se ajustan las coordenadas de peticin, ancho y alto en coordenadas p�xel, 
		//as como la coordenada de inicio de dibujado 
		if(bbox.getULX() > coords[0]) {
			iniX = bbox.getULX();
			x = (int)(((iniX - coords[0]) * w) / Math.abs(coords[2] - coords[0]));
		}
				
		if(bbox.getULY() < coords[1]) {
			iniY = bbox.getULY();
			y = (int)(((coords[1] - iniY) * h) / Math.abs(coords[3] - coords[1]));
		}
		
		if(bbox.getLRX() < coords[2]) 
			endX = bbox.getLRX();
		
		if(bbox.getLRY() > coords[3]) 
			endY = bbox.getLRY();
		
		if(iniX != coords[0] || endX != coords[2]) {
			w = (int)((Math.abs(endX - iniX) * w) / Math.abs(coords[2] - coords[0]));
		}
		
		if(iniY != coords[1] || endY != coords[3]) {
			h = (int)((Math.abs(iniY - endY) * h) / Math.abs(coords[3] - coords[1]));
		}
		
		Extent ext = RasterLocator.getManager().getDataStructFactory().createExtent(iniX, iniY, endX, endY);
		BufferedImage img = throwProcess(ext, w, h);
		g.drawImage(img, x, y, null);
	}
	
	/**
	 * Throws the process sequentially. Since the preview is small,
	 * it should not taken a lot of time
	 * is small
	 * @param ext
	 * @param w
	 * @param h
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private BufferedImage throwProcess(Extent ext, int w, int h) {
		loadLayers();
		
		RasterProcess task;
		try {
			task = RasterBaseAlgorithmLibrary.getManager().createRasterTask("ProjectTemplateProcess");
		} catch (ProcessException e1) {
			e1.printStackTrace();
			return null;
		}
		task.addParam(ProjectTemplateProcess.RASTER_STORE1, inputLyr.getDataStore());
		task.addParam(ProjectTemplateProcess.TEST_EXTENT, ext);
		task.addParam(ProjectTemplateProcess.TEST_WIDTH, w);
		task.addParam(ProjectTemplateProcess.TEST_HEIGHT, h);
		//......
		
		task.init();
		try {
			task.process();
		} catch (ProcessInterruptedException e1) {
			return null;
		} catch (ProcessException e) {
			logger.debug("Error in process");
			return null;
		}
		
		HashMap<String, Object> map = (HashMap<String, Object>)task.getResult();
		Buffer buf = (Buffer)map.get(ProjectTemplateProcess.BUFFER);
		buf.setDataExtent(ext.toRectangle2D());
		if(buf.getBandCount() == 1) {
			buf.replicateBand(0, 1);
			buf.replicateBand(0, 2);
		}
		ImageDrawer drawer = RasterLocator.getManager().createImageDrawerService();
		drawer.setBuffer(buf);
		try {
			//drawer.addEnhanced(inputLyr.getDataStore().getStatistics(), true, 0.2);
			return (BufferedImage)drawer.drawBufferOverImageObject();
		} catch (ProcessInterruptedException e) {
			return null;
		} 
    }
	
	public Object getSource() {
		return null;
	}

	public Rectangle2D getBBoxRectangle() {
		if(inputLyr != null)
			return inputLyr.getFullRasterExtent().toRectangle2D();
		return new Rectangle2D.Double();
	}

	public void addParams(RasterProcess task, ProjectTemplateData data, String path) {
		task.addParam(ProjectTemplateProcess.RASTER_STORE1, inputLyr.getDataStore());
		//......
		task.addParam(ProjectTemplateProcess.PATH, path);
	}
}
