package org.gvsig.raster.projecttemplate.app;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicSplitPaneDivider;
import javax.swing.plaf.basic.BasicSplitPaneUI;

import org.gvsig.andami.PluginServices;
import org.gvsig.andami.ui.mdiManager.IWindow;
import org.gvsig.andami.ui.mdiManager.WindowInfo;
import org.gvsig.gui.beans.buttonspanel.ButtonsPanel;
import org.gvsig.gui.beans.defaultbuttonspanel.DefaultButtonsPanel;

public class ProjectTemplateWindow extends DefaultButtonsPanel implements IWindow, ActionListener {
    
    private static final long  serialVersionUID = -4401123724140025094L;
    private ActionListener     listener         = null;
    private WindowInfo         info             = null;

    private Object             profile = WindowInfo.EDITOR_PROFILE;

    public ProjectTemplateWindow(JComponent panel, 
    		JComponent preview, 
    		String title, 
    		int w, 
    		int h,
    		boolean fixedPreview,
    		ActionListener actionListener) {
    	this.listener = actionListener;
    	
    	if(fixedPreview)
    		fixedPreview(panel, preview);
    	else
    		redimPreview(panel, preview);

        getButtonsPanel().getButton(ButtonsPanel.BUTTON_ACCEPT).addActionListener(this);
        getButtonsPanel().getButton(ButtonsPanel.BUTTON_APPLY).addActionListener(this);
        getButtonsPanel().getButton(ButtonsPanel.BUTTON_CANCEL).addActionListener(this);
        
        info = new WindowInfo(WindowInfo.PALETTE | WindowInfo.RESIZABLE);
        info.setTitle(title);
        info.setWidth(w);
        info.setHeight(h);
    }
    
    private void fixedPreview(JComponent panel, JComponent preview) {
    	setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1;
		gbc.weighty = 1;
		add(panel, gbc);
		
		if(preview != null) {
			gbc.anchor = GridBagConstraints.NORTH;
			gbc.fill = GridBagConstraints.NONE;
			gbc.weightx = 0;
			gbc.weighty = 0;
			gbc.gridx = 1;
			add(preview, gbc);
		}
    }
    
    private void redimPreview(JComponent panel, JComponent preview) {
    	setLayout(new BorderLayout());
		JSplitPane splitPane = new JSplitPane();
		splitPane.setLeftComponent(panel);
		splitPane.setRightComponent(getPreviewForRedimPreviewPanel(preview));
		splitPane.setResizeWeight(1.0);
		splitPane.setContinuousLayout(true);
		splitPane.setOneTouchExpandable(true);
		splitPane.setUI(new BasicSplitPaneUI() {
			public BasicSplitPaneDivider createDefaultDivider() {
				return new BasicSplitPaneDivider(this) {
					private static final long serialVersionUID = 1L;

					public void setBorder(Border b) {
					}
				};
			}
		});
		splitPane.setBorder(null);
		add(splitPane, BorderLayout.CENTER);
    }
    
    private JSplitPane getPreviewForRedimPreviewPanel(JComponent preview) {
    	JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    	splitPane.setTopComponent(preview);
		splitPane.setBottomComponent(new JPanel());
		splitPane.setResizeWeight(0);
		splitPane.setContinuousLayout(true);
		splitPane.setOneTouchExpandable(true);
		splitPane.setUI(new BasicSplitPaneUI() {
			public BasicSplitPaneDivider createDefaultDivider() {
				return new BasicSplitPaneDivider(this) {
					private static final long serialVersionUID = 1L;

					public void setBorder(Border b) {
					}
				};
			}
		});
		splitPane.setBorder(null);
		return splitPane;
    }

    public WindowInfo getWindowInfo() {
        return info;
    }

    public Object getWindowProfile() {
        return profile;
    }

    public void actionPerformed(ActionEvent e) {
    	if(e.getSource() == getButtonsPanel().getButton(ButtonsPanel.BUTTON_ACCEPT)) {
    		listener.actionPerformed(e);
    		PluginServices.getMDIManager().closeWindow(this);
    	}
    	
    	if(e.getSource() == getButtonsPanel().getButton(ButtonsPanel.BUTTON_APPLY)) {
    		listener.actionPerformed(e);
    	}
    	
    	if(e.getSource() == getButtonsPanel().getButton(ButtonsPanel.BUTTON_CANCEL)) {
    		PluginServices.getMDIManager().closeWindow(this);
    	}
    }
}
