package org.gvsig.raster.projecttemplate.lib.impl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Date;
import java.util.regex.Pattern;

import org.gvsig.raster.projecttemplate.lib.ProjectTemplateManager;
import org.gvsig.raster.projecttemplate.lib.ProjectTemplateMessageException;
import org.gvsig.raster.projecttemplate.lib.ProjectTemplateService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Default {@link ProjectTemplateService} implementation.
 * 
 * @author gvSIG Team
 * @version $Id$
 */
public class DefaultProjectTemplateService implements ProjectTemplateService {

    private static final Logger LOG =
        LoggerFactory.getLogger(DefaultProjectTemplateService.class);

    private static final String DEFAULT_FORTUNE_COOKIE_URL =
        "http://www.fullerdata.com/FortuneCookie/FortuneCookie.asmx/GetFortuneCookie";

    private String message = null;
    private ProjectTemplateManager manager;
    private Date date = null;
    private String url;

    /**
     * {@link DefaultProjectTemplateService} constructor with a
     * {@link ProjectTemplateManager}.
     * 
     * @param manager
     *            to use in the service
     */
    public DefaultProjectTemplateService(ProjectTemplateManager manager) {
        this(manager, DEFAULT_FORTUNE_COOKIE_URL);
    }

    /**
     * {@link DefaultProjectTemplateService} constructor with a
     * {@link ProjectTemplateManager}.
     * 
     * @param manager
     *            to use in the service
     */
    public DefaultProjectTemplateService(ProjectTemplateManager manager, String url) {
        this.manager = manager;
        this.url = url;
    }

    public String getMessage() throws ProjectTemplateMessageException {
        if (this.message == null) {
            try {
                this.message = getMessageFromUrl();
            } catch (IOException e) {
                throw new ProjectTemplateMessageException(e);
            }
            this.date = new Date();
        }
        return this.message;
    }

    /**
     * Gets a Raster message from an external service through a URL.
     * 
     * @return the message
     */
    private String getMessageFromUrl() throws IOException {
        LOG.debug("Getting the Raster message from the url: {}",
            this.url);

        URL url = new URL(this.url);
        StringBuffer message = new StringBuffer();
        BufferedReader in =
            new BufferedReader(new InputStreamReader(url.openStream()));
        String str;
        // Create a pattern to match breaks
        Pattern p = Pattern.compile("\\<[^\\>]+\\>");

        StringBuffer data = null;

        if (LOG.isDebugEnabled()) {
            data = new StringBuffer();
        }

        while ((str = in.readLine()) != null) {
            if (LOG.isDebugEnabled()) {
                data.append(str).append('\n');
            }
            message.append(p.matcher(str).replaceAll("\n"));
        }
        in.close();

        LOG.debug("Raster message response:\n{}", data);

        return message.toString();
    }

    public ProjectTemplateManager getManager() {
        return this.manager;
    }

    public Date getDate() {
        return this.date;
    }

}
