package org.gvsig.raster.projecttemplate.swing;

import org.gvsig.tools.locator.BaseLocator;

/**
 * This locator is the entry point for the Raster library, providing
 * access to all Raster services through the {@link ProjectTemplateManager}
 * .
 * 
 * @author gvSIG team
 * @version $Id$
 */
public class ProjectTemplateSwingLocator extends BaseLocator {
    /**
     * Validation swing manager name.
     */
    public static final String SWING_MANAGER_NAME =
        "ProjectTemplate.swing.manager";

    /**
     * Validation swing manager description.
     */
    public static final String SWING_MANAGER_DESCRIPTION =
        "ProjectTemplate UIManager";

    private static final String LOCATOR_NAME = "ProjectTemplate.swing.locator";

    /**
     * Unique instance.
     */
    private static final ProjectTemplateSwingLocator INSTANCE =
        new ProjectTemplateSwingLocator();

    /**
     * Return the singleton instance.
     * 
     * @return the singleton instance
     */
    public static ProjectTemplateSwingLocator getInstance() {
        return INSTANCE;
    }

    /**
     * Return the Locator's name
     * 
     * @return a String with the Locator's name
     */
    public final String getLocatorName() {
        return LOCATOR_NAME;
    }

    /**
     * Registers the Class implementing the PersistenceManager interface.
     * 
     * @param clazz
     *            implementing the PersistenceManager interface
     */
    public static void registerSwingManager(
        Class<? extends ProjectTemplateSwingManager> clazz) {
        getInstance().register(SWING_MANAGER_NAME, SWING_MANAGER_DESCRIPTION, clazz);
    }

    /**
     * Gets the instance of the {@link ScriptingUIManager} registered.
     * 
     * @return {@link ScriptingUIManager}
     */
    public static ProjectTemplateSwingManager getSwingManager() {
        return (ProjectTemplateSwingManager) getInstance().get(SWING_MANAGER_NAME);
    }
}
